@@ -1,19 +1,27 @@
 # package is named tests, not test, so it won't be confused with test in stdlib
 from __future__ import print_function
 
+import contextlib
 import errno
+import functools
 import gc
+import json
 import os
 try:
     import resource
 except ImportError:
     resource = None
 import signal
-import subprocess
+try:
+    import subprocess32 as subprocess  # py2
+except ImportError:
+    import subprocess  # py3
 import sys
 import unittest
 import warnings
 
+from nose.plugins.skip import SkipTest
+
 import eventlet
 from eventlet import tpool
 
@@ -22,22 +30,29 @@ from eventlet import tpool
 main = unittest.main
 
 
-def skipped(func):
-    """ Decorator that marks a function as skipped.  Uses nose's SkipTest exception
-    if installed.  Without nose, this will count skipped tests as passing tests."""
+@contextlib.contextmanager
+def assert_raises(exc_type):
     try:
-        from nose.plugins.skip import SkipTest
+        yield
+    except exc_type:
+        pass
+    else:
+        name = str(exc_type)
+        try:
+            name = exc_type.__name__
+        except AttributeError:
+            pass
+        assert False, 'Expected exception {0}'.format(name)
 
-        def skipme(*a, **k):
-            raise SkipTest()
-        skipme.__name__ = func.__name__
-        return skipme
-    except ImportError:
-        # no nose, we'll just skip the test ourselves
-        def skipme(*a, **k):
-            print(("Skipping {0}".format(func.__name__)))
-        skipme.__name__ = func.__name__
-        return skipme
+
+def skipped(func, *decorator_args):
+    """Decorator that marks a function as skipped.
+    """
+    @functools.wraps(func)
+    def wrapped(*a, **k):
+        raise SkipTest(*decorator_args)
+
+    return wrapped
 
 
 def skip_if(condition):
@@ -47,16 +62,16 @@ def skip_if(condition):
     should return True to skip the test.
     """
     def skipped_wrapper(func):
+        @functools.wraps(func)
         def wrapped(*a, **kw):
             if isinstance(condition, bool):
                 result = condition
             else:
                 result = condition(func)
             if result:
-                return skipped(func)(*a, **kw)
+                raise SkipTest()
             else:
                 return func(*a, **kw)
-        wrapped.__name__ = func.__name__
         return wrapped
     return skipped_wrapper
 
@@ -68,16 +83,16 @@ def skip_unless(condition):
     should return True if the condition is satisfied.
     """
     def skipped_wrapper(func):
+        @functools.wraps(func)
         def wrapped(*a, **kw):
             if isinstance(condition, bool):
                 result = condition
             else:
                 result = condition(func)
             if not result:
-                return skipped(func)(*a, **kw)
+                raise SkipTest()
             else:
                 return func(*a, **kw)
-        wrapped.__name__ = func.__name__
         return wrapped
     return skipped_wrapper
 
@@ -197,6 +212,13 @@ def check_idle_cpu_usage(duration, allowed_part):
     r2 = resource.getrusage(resource.RUSAGE_SELF)
     utime = r2.ru_utime - r1.ru_utime
     stime = r2.ru_stime - r1.ru_stime
+
+    # This check is reliably unreliable on Travis, presumably because of CPU
+    # resources being quite restricted by the build environment. The workaround
+    # is to apply an arbitrary factor that should be enough to make it work nicely.
+    if os.environ.get('TRAVIS') == 'true':
+        allowed_part *= 1.2
+
     assert utime + stime < duration * allowed_part, \
         "CPU usage over limit: user %.0f%% sys %.0f%% allowed %.0f%%" % (
             utime / duration * 100, stime / duration * 100,
@@ -204,13 +226,23 @@ def check_idle_cpu_usage(duration, allowed_part):
 
 
 def verify_hub_empty():
+
+    def format_listener(listener):
+        return 'Listener %r for greenlet %r with run callback %r' % (
+            listener, listener.greenlet, getattr(listener.greenlet, 'run', None))
+
     from eventlet import hubs
     hub = hubs.get_hub()
-    num_readers = len(hub.get_readers())
-    num_writers = len(hub.get_writers())
+    readers = hub.get_readers()
+    writers = hub.get_writers()
+    num_readers = len(readers)
+    num_writers = len(writers)
     num_timers = hub.get_timers_count()
-    assert num_readers == 0 and num_writers == 0, "Readers: %s Writers: %s" % (
-        num_readers, num_writers)
+    assert num_readers == 0 and num_writers == 0, \
+        "Readers: %s (%d) Writers: %s (%d)" % (
+            ', '.join(map(format_listener, readers)), num_readers,
+            ', '.join(map(format_listener, writers)), num_writers,
+        )
 
 
 def find_command(command):
@@ -245,19 +277,10 @@ def get_database_auth():
     ".test_dbauth", which contains a json map of parameters to the
     connect function.
     """
-    import os
     retval = {
         'MySQLdb': {'host': 'localhost', 'user': 'root', 'passwd': ''},
         'psycopg2': {'user': 'test'},
     }
-    try:
-        import json
-    except ImportError:
-        try:
-            import simplejson as json
-        except ImportError:
-            print("No json implementation, using baked-in db credentials.")
-            return retval
 
     if 'EVENTLET_DB_TEST_AUTH' in os.environ:
         return json.loads(os.environ.get('EVENTLET_DB_TEST_AUTH'))
@@ -268,7 +291,7 @@ def get_database_auth():
         try:
             auth_utf8 = json.load(open(f))
             # Have to convert unicode objects to str objects because
-            # mysqldb is dum. Using a doubly-nested list comprehension
+            # mysqldb is dumb. Using a doubly-nested list comprehension
             # because we know that the structure is a two-level dict.
             return dict(
                 [(str(modname), dict(
@@ -279,23 +302,79 @@ def get_database_auth():
     return retval
 
 
-def run_python(path):
-    if not path.endswith('.py'):
-        path += '.py'
-    path = os.path.abspath(path)
-    dir_ = os.path.dirname(path)
+def run_python(path, env=None, args=None, timeout=None, pythonpath_extend=None, expect_pass=False):
+    new_argv = [sys.executable]
+    if sys.version_info[:2] <= (2, 6):
+        new_argv += ['-W', 'ignore::DeprecationWarning']
     new_env = os.environ.copy()
-    new_env['PYTHONPATH'] = os.pathsep.join(sys.path + [dir_])
+    new_env.setdefault('eventlet_test_in_progress', 'yes')
+    src_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
+    if path:
+        path = os.path.abspath(path)
+        new_argv.append(path)
+        new_env['PYTHONPATH'] = os.pathsep.join(sys.path + [src_dir])
+    if env:
+        new_env.update(env)
+    if pythonpath_extend:
+        new_path = [p for p in new_env.get('PYTHONPATH', '').split(os.pathsep) if p]
+        new_path.extend(
+            p if os.path.isabs(p) else os.path.join(src_dir, p) for p in pythonpath_extend
+        )
+        new_env['PYTHONPATH'] = os.pathsep.join(new_path)
+    if args:
+        new_argv.extend(args)
     p = subprocess.Popen(
-        [sys.executable, path],
+        new_argv,
         env=new_env,
         stderr=subprocess.STDOUT,
         stdin=subprocess.PIPE,
         stdout=subprocess.PIPE,
     )
-    output, _ = p.communicate()
+    if timeout is None:
+        timeout = 10
+    try:
+        output, _ = p.communicate(timeout=timeout)
+    except subprocess.TimeoutExpired:
+        p.kill()
+        output, _ = p.communicate(timeout=timeout)
+        return '{0}\nFAIL - timed out'.format(output).encode()
+
+    if expect_pass:
+        if output.startswith(b'skip'):
+            parts = output.rstrip().split(b':', 1)
+            skip_args = []
+            if len(parts) > 1:
+                skip_args.append(parts[1])
+            raise SkipTest(*skip_args)
+        ok = output.rstrip() == b'pass'
+        if not ok:
+            sys.stderr.write('Program {0} output:\n---\n{1}\n---\n'.format(path, output.decode()))
+        assert ok, 'Expected single line "pass" in stdout'
+
     return output
 
 
+def run_isolated(path, prefix='tests/isolated/', **kwargs):
+    kwargs.setdefault('expect_pass', True)
+    run_python(prefix + path, **kwargs)
+
+
+def check_is_timeout(obj):
+    value_text = getattr(obj, 'is_timeout', '(missing)')
+    assert obj.is_timeout, 'type={0} str={1} .is_timeout={2}'.format(type(obj), str(obj), value_text)
+
+
 certificate_file = os.path.join(os.path.dirname(__file__), 'test_server.crt')
 private_key_file = os.path.join(os.path.dirname(__file__), 'test_server.key')
+
+
+def test_run_python_timeout():
+    output = run_python('', args=('-c', 'import time; time.sleep(0.5)'), timeout=0.1)
+    assert output.endswith(b'FAIL - timed out')
+
+
+def test_run_python_pythonpath_extend():
+    code = '''import os, sys ; print('\\n'.join(sys.path))'''
+    output = run_python('', args=('-c', code), pythonpath_extend=('dira', 'dirb'))
+    assert b'/dira\n' in output
+    assert b'/dirb\n' in output
