@@ -4,15 +4,12 @@ import sys
 import tempfile
 
 from eventlet.support import six
-from tests import LimitedTestCase, main, run_python, skip_with_pyevent
+import tests
 
 
 base_module_contents = """
 import socket
-try:
-    import urllib.request as urllib
-except ImportError:
-    import urllib
+import urllib
 print("base {0} {1}".format(socket, urllib))
 """
 
@@ -32,7 +29,7 @@ print("importing {0} {1} {2} {3}".format(patching, socket, patching.socket, patc
 """
 
 
-class ProcessBase(LimitedTestCase):
+class ProcessBase(tests.LimitedTestCase):
     TEST_TIMEOUT = 3  # starting processes is time-consuming
 
     def setUp(self):
@@ -54,7 +51,7 @@ class ProcessBase(LimitedTestCase):
 
     def launch_subprocess(self, filename):
         path = os.path.join(self.tempdir, filename)
-        output = run_python(path)
+        output = tests.run_python(path)
         if six.PY3:
             output = output.decode('utf-8')
             separator = '\n'
@@ -85,19 +82,9 @@ class ImportPatched(ProcessBase):
         assert 'eventlet.green.urllib' in lines[2], repr(output)
         assert 'eventlet.green.httplib' not in lines[2], repr(output)
 
-    def test_import_patched_defaults(self):
-        self.write_to_tempfile("base", base_module_contents)
-        new_mod = """
-from eventlet import patcher
-base = patcher.import_patched('base')
-print("newmod {0} {1} {2}".format(base, base.socket, base.urllib.socket.socket))
-"""
-        self.write_to_tempfile("newmod", new_mod)
-        output, lines = self.launch_subprocess('newmod.py')
-        assert lines[0].startswith('base'), repr(output)
-        assert lines[1].startswith('newmod'), repr(output)
-        assert 'eventlet.green.socket' in lines[1], repr(output)
-        assert 'GreenSocket' in lines[1], repr(output)
+
+def test_import_patched_defaults():
+    tests.run_isolated('patcher_import_patched_defaults.py')
 
 
 class MonkeyPatch(ProcessBase):
@@ -181,15 +168,15 @@ print("already_patched {0}".format(",".join(sorted(patcher.already_patched.keys(
 
     def test_boolean(self):
         self.assert_boolean_logic("patcher.monkey_patch()",
-                                  'os,select,socket,thread,time')
+                                  'os,select,socket,subprocess,thread,time')
 
     def test_boolean_all(self):
         self.assert_boolean_logic("patcher.monkey_patch(all=True)",
-                                  'os,select,socket,thread,time')
+                                  'os,select,socket,subprocess,thread,time')
 
     def test_boolean_all_single(self):
         self.assert_boolean_logic("patcher.monkey_patch(all=True, socket=True)",
-                                  'os,select,socket,thread,time')
+                                  'os,select,socket,subprocess,thread,time')
 
     def test_boolean_all_negative(self):
         self.assert_boolean_logic(
@@ -206,11 +193,11 @@ print("already_patched {0}".format(",".join(sorted(patcher.already_patched.keys(
 
     def test_boolean_negative(self):
         self.assert_boolean_logic("patcher.monkey_patch(socket=False)",
-                                  'os,select,thread,time')
+                                  'os,select,subprocess,thread,time')
 
     def test_boolean_negative2(self):
         self.assert_boolean_logic("patcher.monkey_patch(socket=False, time=False)",
-                                  'os,select,thread')
+                                  'os,select,subprocess,thread')
 
     def test_conflicting_specifications(self):
         self.assert_boolean_logic("patcher.monkey_patch(socket=False, select=True)",
@@ -242,7 +229,7 @@ def test_monkey_patch_threading():
 class Tpool(ProcessBase):
     TEST_TIMEOUT = 3
 
-    @skip_with_pyevent
+    @tests.skip_with_pyevent
     def test_simple(self):
         new_mod = """
 import eventlet
@@ -260,7 +247,7 @@ tpool.killall()
         assert '2' in lines[0], repr(output)
         assert '3' in lines[1], repr(output)
 
-    @skip_with_pyevent
+    @tests.skip_with_pyevent
     def test_unpatched_thread(self):
         new_mod = """import eventlet
 eventlet.monkey_patch(time=False, thread=False)
@@ -273,7 +260,7 @@ import time
         output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 2, lines)
 
-    @skip_with_pyevent
+    @tests.skip_with_pyevent
     def test_patched_thread(self):
         new_mod = """import eventlet
 eventlet.monkey_patch(time=False, thread=True)
@@ -287,18 +274,20 @@ import time
         self.assertEqual(len(lines), 2, "\n".join(lines))
 
 
-class Subprocess(ProcessBase):
-    def test_monkeypatched_subprocess(self):
-        new_mod = """import eventlet
+def test_subprocess_after_monkey_patch():
+    code = '''\
+import sys
+import eventlet
 eventlet.monkey_patch()
 from eventlet.green import subprocess
-
-subprocess.Popen(['true'], stdin=subprocess.PIPE)
-print("done")
-"""
-        self.write_to_tempfile("newmod", new_mod)
-        output, lines = self.launch_subprocess('newmod')
-        self.assertEqual(output, "done\n", output)
+subprocess.Popen([sys.executable, '-c', ''], stdin=subprocess.PIPE).wait()
+print('pass')
+'''
+    output = tests.run_python(
+        path=None,
+        args=['-c', code],
+    )
+    assert output.rstrip() == b'pass'
 
 
 class Threading(ProcessBase):
@@ -317,11 +306,11 @@ print(len(threading._active))
 print(len(_threading._active))
 """
         self.write_to_tempfile("newmod", new_mod)
-        output, lines = self.launch_subprocess('newmod')
+        output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 4, "\n".join(lines))
         assert lines[0].startswith('<Thread'), lines[0]
-        self.assertEqual(lines[1], "1", lines[1])
-        self.assertEqual(lines[2], "1", lines[2])
+        assert lines[1] == '1', lines
+        assert lines[2] == '1', lines
 
     def test_threading(self):
         new_mod = """import eventlet
@@ -335,7 +324,7 @@ t.join()
 print(len(threading._active))
 """
         self.write_to_tempfile("newmod", new_mod)
-        output, lines = self.launch_subprocess('newmod')
+        output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 3, "\n".join(lines))
         assert lines[0].startswith('<_MainThread'), lines[0]
         self.assertEqual(lines[1], "1", lines[1])
@@ -351,7 +340,7 @@ tpool.execute(test)
 print(len(threading._active))
 """
         self.write_to_tempfile("newmod", new_mod)
-        output, lines = self.launch_subprocess('newmod')
+        output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 3, "\n".join(lines))
         assert lines[0].startswith('<Thread'), lines[0]
         self.assertEqual(lines[1], "1", lines[1])
@@ -370,7 +359,7 @@ evt.wait()
 print(len(threading._active))
 """
         self.write_to_tempfile("newmod", new_mod)
-        output, lines = self.launch_subprocess('newmod')
+        output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 3, "\n".join(lines))
         assert lines[0].startswith('<_MainThread'), lines[0]
         self.assertEqual(lines[1], "1", lines[1])
@@ -386,7 +375,7 @@ t.wait()
 print(len(threading._active))
 """
         self.write_to_tempfile("newmod", new_mod)
-        output, lines = self.launch_subprocess('newmod')
+        output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 3, "\n".join(lines))
         assert lines[0].startswith('<_GreenThread'), lines[0]
         self.assertEqual(lines[1], "1", lines[1])
@@ -396,7 +385,7 @@ print(len(threading._active))
 eventlet.monkey_patch()
 """
         self.write_to_tempfile("newmod", new_mod)
-        output, lines = self.launch_subprocess('newmod')
+        output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 1, "\n".join(lines))
 
 
@@ -408,7 +397,7 @@ eventlet.monkey_patch(all=False, os=True)
 process = subprocess.Popen("sleep 0.1 && false", shell=True)
 print(process.wait())"""
         self.write_to_tempfile("newmod", new_mod)
-        output, lines = self.launch_subprocess('newmod')
+        output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 2, "\n".join(lines))
         self.assertEqual('1', lines[0], repr(output))
 
@@ -435,7 +424,7 @@ t.wait()
 print(repr(t2))
 t2.join()
 """)
-        output, lines = self.launch_subprocess('newmod')
+        output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 2, "\n".join(lines))
         assert lines[0].startswith('<_GreenThread'), lines[0]
 
@@ -453,7 +442,7 @@ t2.join()
     print(t.getName())
     print(t.get_name())
 """ + self.epilogue)
-        output, lines = self.launch_subprocess('newmod')
+        output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 10, "\n".join(lines))
         for i in range(0, 3):
             self.assertEqual(lines[i], "GreenThread-1", lines[i])
@@ -467,7 +456,7 @@ t2.join()
     print(id(t._g))
     print(t.ident)
 """ + self.epilogue)
-        output, lines = self.launch_subprocess('newmod')
+        output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 3, "\n".join(lines))
         self.assertEqual(lines[0], lines[1])
 
@@ -476,7 +465,7 @@ t2.join()
     print(t.is_alive())
     print(t.isAlive())
 """ + self.epilogue)
-        output, lines = self.launch_subprocess('newmod')
+        output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 3, "\n".join(lines))
         self.assertEqual(lines[0], "True", lines[0])
         self.assertEqual(lines[1], "True", lines[1])
@@ -486,16 +475,43 @@ t2.join()
     print(t.is_daemon())
     print(t.isDaemon())
 """ + self.epilogue)
-        output, lines = self.launch_subprocess('newmod')
+        output, lines = self.launch_subprocess('newmod.py')
         self.assertEqual(len(lines), 3, "\n".join(lines))
         self.assertEqual(lines[0], "True", lines[0])
         self.assertEqual(lines[1], "True", lines[1])
 
 
+def test_patcher_existing_locks_early():
+    tests.run_isolated('patcher_existing_locks_early.py')
+
+
+def test_patcher_existing_locks_late():
+    tests.run_isolated('patcher_existing_locks_late.py')
+
+
+def test_patcher_existing_locks_locked():
+    tests.run_isolated('patcher_existing_locks_locked.py')
+
+
 def test_importlib_lock():
-    output = run_python('tests/patcher_test_importlib_lock.py')
-    assert output.rstrip() == b'ok'
+    tests.run_isolated('patcher_importlib_lock.py')
 
 
-if __name__ == '__main__':
-    main()
+def test_threading_condition():
+    tests.run_isolated('patcher_threading_condition.py')
+
+
+def test_threading_join():
+    tests.run_isolated('patcher_threading_join.py')
+
+
+def test_socketserver_selectors():
+    tests.run_isolated('patcher_socketserver_selectors.py')
+
+
+def test_blocking_select_methods_are_deleted():
+    tests.run_isolated('patcher_blocking_select_methods_are_deleted.py')
+
+
+def test_regular_file_readall():
+    tests.run_isolated('regular_file_readall.py')
